/* This file is under the LGPL license */

#include <stdio.h>
#include <getopt.h>
#include <string.h>
#include <stdlib.h>
#include <eci.h>


static void say_text(char *fileName, char* text, int speed, int rate) {
  ECIHand aHandle = eciNew();

  if(fileName != NULL) {
      eciSetOutputFilename(aHandle, fileName);
  }
  eciSetParam(aHandle, eciSampleRate + 1, 2);
  if (aHandle == NULL_ECI_HAND) {
      printf( "Error eciNew\n" );
  } else if ((eciSetParam(aHandle, eciInputType, 1) == -1)
	   || (eciSetParam(aHandle, eciSynthMode, 1) == -1)
	   || (eciSetParam(aHandle, eciSampleRate, rate) == -1)) {
      printf( "Error eciSetParam\n" );
  } else if (eciSetVoiceParam(aHandle, 0, eciSpeed, speed) == -1) {
      printf( "Bad speed\n" );
  } else if (!eciAddText(aHandle, text)) {
      printf( "Error eciAddText\n" );
  } else if (!eciSynthesize(aHandle)) {
      printf( "Error eciSynthesize\n" );
  } else if (!eciSynchronize(aHandle)) {
      printf( "Error eciSynchronize\n" );
  }
  eciDelete(aHandle);
}

#define BUFFERSIZE 100
static short buffer[BUFFERSIZE];

/* Read in the file and return it. */
static char *readFile(
    char *fileName)
{
    unsigned int pos = 0;
    unsigned int size = 42;
    char *buffer = (char *)calloc(size, sizeof(char));
    FILE *file = fopen(fileName, "r");
    int c;

    if(file == NULL) {
	printf("Unable to open file %s\n", fileName);
	exit(1);
    }
    while((c = getc(file)) && c != EOF) {
	if(pos + 1 == size) {
	    size += size >> 1;
	    buffer = (char *)realloc(buffer, size);
	}
	buffer[pos++] = c;
    }
    buffer[pos] = '\0';
    fclose(file);
    return buffer;
}

int main(int argc, char *argv[]) {
    char* wavfile = NULL;
    char* sentence = NULL;
    int opt;
    int speed = 50;
    int rate = 2; /* 0 = 8KHz, 1 = 11.025KHz, 2 = 22.05KHz */
    char *fileName = NULL;

    while ((opt = getopt(argc, argv, "w:s:r:f:")) != -1) {
        switch (opt) {
        case 'w':
            wavfile = strdup(optarg);
            break;
        case 's':
            speed = atoi(optarg);
            break;
        case 'r':
            rate = atoi(optarg);
            break;
        case 'f':
            fileName = strdup(optarg);
            break;
        default: /* ’?’ */
            fprintf(stderr, "Usage: %s [-w wavfile] [-s speed] [-f text file] [-r sample-rate] [text]\n"
                    "        The sample rate is 0 for 8KHz, 1 for 11KHz, and 2 foor 22KHz\n",
	            argv[0]);
            return(1);
        }
    }

    if (optind == argc-1 && fileName == NULL) {
        sentence = strdup(argv[optind]);
    } else if(fileName != NULL) {
        sentence = readFile(fileName);
    } else {
        sentence = strdup("Hello World!");
    }
    if(wavfile != NULL) {
        unlink(wavfile);
    }
    say_text(wavfile, sentence, speed, rate);
    return 0;
}
